div.col-md-9 img[alt="Arpeggio logo"] { background-color: transparent; border: none; }

div.col-md-9 h1:first-of-type {
    text-align: center;
    font-size: 60px;
    font-weight: 300;
}

div.col-md-9>p:first-of-type {
    text-align: center;
}

div.col-md-9 p.admonition-title:first-of-type {
    text-align: left;
}

div.col-md-9 h1:first-of-type .headerlink {
    display: none;
}
